function [ff] = nonlinnew(pp,params,pi,rk,country,Taylor,forward,Re,c,i,g,R,Ra)
%
% These are the nonlinear equations of the CMR model for checking the linearisations done by hand and making higher order expansions.
%
%  30 equations in the baseline model - 
%  Equations in wage and price section: (2), (3), (31), (32), (33), (34), (35), (36); 8 total
%                        Capital producer section: (4), (21); 2 total
%                        Entrepreneur (5), (6), (7), (8), (9); 5 total
%                        Banking, (10), (11), (12), (13), (26); 5 total
%                        Households, (14), (15), (16), (17), (18), (24), (27); 6 of these in baseline
%                        Resource Constraint, (20); 1
%                        equation defining M3, (28); 1
%                        equation defining total loans, (30); 1
%                        Law of motion relating the base to its growth rate, (22); 1
% missing equations: previous price equation - (1); previous wage equation - (19);
%                        equation relating the base growth rate to exogenous shocks - (23);
%                        equation (25) - not used.
%                        equation (29) - not used here....a definitional equation in the basic system
%                        equation (31) - not used here....a definitional equation
%

ff=zeros(38,1);
entr      = params(1);
pref      = params(2);
lambdaw   = params(3);
sigmaL    = params(4);
theta     = params(5);
v         = params(6);
beta      = params(7);
xiw       = params(8);
Hdoupr    = params(9);
sigmaq    = params(10);
b         = params(11);
psiL      = params(12);
m         = params(13);
chii      = params(14);
duration1 = params(15);
zetac     = params(16);
iotaw1    = params(17);
iotaw2    = params(18);
iotamu    = params(19);
Fomegabar = params(20);
sigma     = params(21);
mu        = params(22);
gamma     = params(23);
we        = params(24);
bigtheta  = params(25);
sigmaa    = params(26);
Sdoupr    = params(27);
etai      = params(28);
muup      = params(29);
zeta      = params(30);
muzstar   = params(31);
upsil     = params(32);
lambdaf   = params(33);
alpha     = params(34);
psik      = params(35);
psil      = params(36);
delta     = params(37);
epsil     = params(38);
xip       = params(39);
iota1     = params(40);
iota2     = params(41);
ksi       = params(42);
xb        = params(43);
chorse    = params(44);
rhobank   = params(45);
vl        = params(46);
vk        = params(47);
etag      = params(48);
tauk      = params(49);
taul      = params(50);
tauc      = params(51);
taud      = params(52);
taut      = params(53);
tau       = params(54);
x         = params(55);
aptil     = params(56);
aytil     = params(57);
amtil     = params(58);
actil     = params(59);
rhotil    = params(60);
timing    = params(61);
nn        = params(62);
xp        = params(63);
phi       = params(64);
pibar     = params(65);


nn=nn+5; %the non-linear price and wage equations require adding 4 new variables; then add wplus (i.e. they become 33 when entr=1)

zp1=pp([1:nn]);
z0=pp([nn+1:2*nn]);
zm1=pp([2*nn+1:3*nn]);
psip1=pp([3*nn+1:3*nn+25]);
psip0=pp([3*nn+25+1:end]);

%
pip1=zp1(1);
sp1=zp1(2);
rkp1=zp1(3);
ip1=zp1(4);
up1=zp1(5);
if entr == 1
    omegabarp1=zp1(6);
    Rkp1=zp1(7);
    np2=zp1(8);
    qp1=zp1(9);
    vlp1=zp1(10);
    evp1=zp1(11);
    ii=11;
elseif entr == 0
    Rkp1=zp1(6);
    qp1=zp1(7);
    vlp1=zp1(8);
    evp1=zp1(9);
    ii=9;
elseif entr <= -1
    Rkp1=zp1(6);
    qp1=zp1(7);
    ii=7;
end
if entr > -2
    mbp2=zp1(ii+1);
    ii=ii+1;
end
Rp1=zp1(ii+1);
uzcp1=zp1(ii+2);
if entr > -2
    lambdazp1=zp1(ii+3);
    mp1=zp1(ii+4);
    ii=ii+4;
else
    ii=ii+2;
end
if entr>-1
    Rap1=zp1(ii+1);
    if entr ==1
        iii=17;
    elseif entr ==0
        iii=15;
    end
elseif entr==-1
    iii=12;
elseif entr == -2
    iii=9;
end
cp1=zp1(iii+1);
wp1=zp1(iii+2);
hp1=zp1(iii+3);
kbarp2=zp1(iii+4);
if entr > -1
    Rep2=zp1(iii+5);
end
if entr > -1
    iii=iii+5;
elseif entr <= -1
    iii=iii+4;
end
if entr > -2
    xp1=zp1(iii+1);
    iii=iii+1;
end
if entr>-1
    dmp2=zp1(iii+1);
    Rmp2=zp1(iii+2);
    m3p1=zp1(iii+3);
    mbp1=zp1(iii+4);
    iii=iii+4;
end
if entr==1
    btotp1=zp1(iii+1);
    iii=iii+1;
end    
if entr>-1
    kbarp1=zp1(iii+1);
    iii=iii+1;
end
pstarp1=zp1(iii+1);
wstarp1=zp1(iii+2);
Fpp1=zp1(iii+3);
Fwp1=zp1(iii+4);
wplusp1=zp1(iii+5);
%
pi0=z0(1);
s0=z0(2);
rk0=z0(3);
i0=z0(4);
u0=z0(5);
if entr == 1
    omegabar0=z0(6);
    Rk0=z0(7);
    np1=z0(8);
    q0=z0(9);
    vl0=z0(10);
    ev0=z0(11);
    ii=11;
elseif entr == 0
    Rk0=z0(6);
    q0=z0(7);
    vl0=z0(8);
    ev0=z0(9);
    ii=9;
elseif entr <= -1
    Rk0=z0(6);
    q0=z0(7);
    ii=7;
end
if entr > -2
    mbp1=z0(ii+1);
    ii=ii+1;
end
R0=z0(ii+1);
uzc0=z0(ii+2);
if entr > -2
    lambdaz0=z0(ii+3);
    m0=z0(ii+4);
    ii=ii+4;
else
    ii=ii+2;
end
if entr>-1
    Ra0=z0(ii+1);
    if entr ==1
        iii=17;
    elseif entr ==0
        iii=15;
    end
elseif entr==-1
    iii=12;
elseif entr == -2
    iii=9;
end
c0=z0(iii+1);
w0=z0(iii+2);
h0=z0(iii+3);
kbarp1=z0(iii+4);
if entr > -1
    Rep1=z0(iii+5);
end
if entr > -1
    iii=iii+5;
elseif entr <= -1
    iii=iii+4;
end
if entr > -2
    x0=z0(iii+1);
    iii=iii+1;
end
if entr>-1
    dmp1=z0(iii+1);
    Rmp1=z0(iii+2);
    m30=z0(iii+3);
    mb0=z0(iii+4);
    iii=iii+4;
end
if entr==1
    btot0=z0(iii+1);
    iii=iii+1;
end    
if entr>-1
    kbar0=z0(iii+1);
    iii=iii+1;
end
pstar0=z0(iii+1);
wstar0=z0(iii+2);
Fp0=z0(iii+3);
Fw0=z0(iii+4);
wplus0=z0(iii+5);
%
pim1=zm1(1);
sm1=zm1(2);
rkm1=zm1(3);
im1=zm1(4);
um1=zm1(5);
if entr == 1
    omegabarm1=zm1(6);
    Rkm1=zm1(7);
    n0=zm1(8);
    qm1=zm1(9);
    vlm1=zm1(10);
    evm1=zm1(11);
    ii=11;
elseif entr == 0
    Rkm1=zm1(6);
    qm1=zm1(7);
    vlm1=zm1(8);
    evm1=zm1(9);
    ii=9;
elseif entr <= -1
    Rkm1=zm1(6);
    qm1=zm1(7);
    ii=7;
end
if entr > -2
    mb0=zm1(ii+1);
    ii=ii+1;
end
Rm1=zm1(ii+1);
uzcm1=zm1(ii+2);
if entr > -2
    lambdazm1=zm1(ii+3);
    mm1=zm1(ii+4);
    ii=ii+4;
else
    ii=ii+2;
end
if entr>-1
    Ram1=zm1(ii+1);
    if entr ==1
        iii=17;
    elseif entr ==0
        iii=15;
    end
elseif entr==-1
    iii=12;
elseif entr == -2
    iii=9;
end
cm1=zm1(iii+1);
wm1=zm1(iii+2);
hm1=zm1(iii+3);
kbar0=zm1(iii+4);
if entr > -1
    Re0=zm1(iii+5);
end
if entr > -1
    iii=iii+5;
elseif entr <= -1
    iii=iii+4;
end
if entr > -2
    xm1=zm1(iii+1);
    iii=iii+1;
end
if entr>-1
    dm0=zm1(iii+1);
    Rm0=zm1(iii+2);
    m3m1=zm1(iii+3);
    mbm1=zm1(iii+4);
    iii=iii+4;
end
if entr==1
    btotm1=zm1(iii+1);
    iii=iii+1;
end    
if entr>-1
    kbarm1=zm1(iii+1);
    iii=iii+1;
end
pstarm1=zm1(iii+1);
wstarm1=zm1(iii+2);
Fpm1=zm1(iii+3);
Fwm1=zm1(iii+4);
wplusm1=zm1(iii+5);

%
lambdafp1=psip1(1);
pitargetp1=psip1(2);
psilp1=psip1(3);
psikp1=psip1(4);
ksip1=psip1(5);
xbp1=psip1(6);
muupp1=psip1(7);
thetap1=psip1(8);
chiip1=psip1(9);
taulp1=psip1(10);
taukp1=psip1(11);
zetap1=psip1(12);
gp1=psip1(13);
vp1=psip1(14);
taudp1=psip1(15);
muzstarp1=psip1(16);
gammap1=psip1(17);
epsilp1=psip1(18);
xpp1=psip1(19);
taucp1=psip1(20);
sigmap1=psip1(21);
zetacp1=psip1(22);
zetaip1=psip1(23);
tauop1=psip1(24);
termp1=psip1(25);

%
lambdaf0=psip0(1);
lambdafm1=psip0(2);
pitarget0=psip0(3);
pitargetm1=psip0(4);
psil0=psip0(5);
psilm1=psip0(6);
psik0=psip0(7);
psikm1=psip0(8);
ksi0=psip0(9);
ksim1=psip0(10);
xb0=psip0(11);
xbm1=psip0(12);
muup0=psip0(13);
muupm1=psip0(14);
theta0=psip0(15);
thetam1=psip0(16);
chii0=psip0(17);
chiim1=psip0(18);
taul0=psip0(19);
taulm1=psip0(20);
tauk0=psip0(21);
taukm1=psip0(22);
zeta0=psip0(23);
zetam1=psip0(24);
g0=psip0(25);
gm1=psip0(26);
v0=psip0(27);
vm1=psip0(28);
taud0=psip0(29);
taudm1=psip0(30);
muzstar0=psip0(31);
muzstarm1=psip0(32);
gamma0=psip0(33);
gammam1=psip0(34);
epsil0=psip0(35);
epsilm1=psip0(36);
xp0=psip0(37);
xpm1=psip0(38);
tauc0=psip0(39);
taucm1=psip0(40);
sigma0=psip0(41);
sigmam1=psip0(42);
zetac0=psip0(43);
zetacm1=psip0(44);
zetai0=psip0(45);
zetaim1=psip0(46);
tauo0=psip0(47);
tauom1=psip0(48);
term0=psip0(49);
termm1=psip0(50);

if timing == 0
    error('fatal (nonlin) timing=0 not allowed')
end

%-------------------------------------------------------------------------
%-------------------------------------------------------------------------
%                                 start list of equations
begin_here=1; %this is a flag needed by the program transform2dynare.m to know where the list of equations starts. Do not change it!
%-------------------------------------------------------------------------
%-------------------------------------------------------------------------


%h denotes household aggregate differentiated labor supply, while l denotes aggregate homogeneous labor
lp1=hp1*(wstarp1 ^ (lambdaw/(lambdaw-1)) );
l0=h0*(wstar0 ^ (lambdaw/(lambdaw-1)) );
lm1=hm1*(wstarm1 ^ (lambdaw/(lambdaw-1)) );
if entr >-1
    Yz0 = (pstar0^(lambdaf0/(lambdaf0-1))) * ( epsil0 * vl0 * ( (u0*kbar0/(muzstar0*upsil))^alpha ) * ( l0^(1-alpha) ) - phi );
elseif entr <=-1
    Yz0 = (pstar0^(lambdaf0/(lambdaf0-1))) * ( epsil0 * (( u0*kbar0/(muzstar0*upsil))^alpha ) * ( l0^(1-alpha) ) - phi);
end

%when entr == -2, we drop equation 18 and lambdaz as an unknown in the z(t) vector. 
if entr == -2
    lambdaz0=uzc0/(zetac0*(1+tauc0));
    lambdazp1=uzcp1/(zetacp1*(1+taucp1));
    lambdazm1=uzcm1/(zetacm1*(1+taucm1));
end

%one efficiency condition for intermediate good firm
ff(2)=-s0+((1-alpha)^(alpha-1)*alpha^(-alpha))*((rk0*(1+psik0*R0))^alpha)*((w0*(1+psil0*R0))^(1-alpha))/epsil0;
   
%second efficiency condition for intermediate good firm:
ff(3)=rk0*(1+psik0*R0)/( alpha*epsil0*( (upsil*muzstar0*l0/(u0*kbar0))^(1-alpha) ) )-s0;

I0Im1=zetai0*i0*muzstar0*upsil/im1;
A=sqrt(Sdoupr/2);
S0=exp(A*(I0Im1-muzstar*upsil))+exp(-A*(I0Im1-muzstar*upsil))-2;
dS0=A*exp(A*(I0Im1-muzstar*upsil))-A*exp(-A*(I0Im1-muzstar*upsil));
F10=-dS0*I0Im1+1-S0;
Ip1I0=zetaip1*ip1*muzstarp1*upsil/i0;
dSp1=A*exp(A*(Ip1I0-muzstar*upsil))-A*exp(-A*(Ip1I0-muzstar*upsil));
F2p1=dSp1*(Ip1I0^2);
%efficiency condition for capital producers
ff(4)=lambdaz0*zetac0*q0*F10-(lambdaz0/muup0)*zetac0+beta*zetacp1*lambdazp1*qp1*F2p1/(muzstarp1*upsil);

% eq. 5: FOC for capital utilisation
ff(5)= rk0 - tauo0*rk*exp(sigmaa*(u0-1));


%
if entr == 1
    ssp1=(1+Rkp1)/(1+Rep1);
    %[Gamma0,Gam_muG0] = helpd(omegabar0,mu,sigmam1);
    xzx0       =   (log(omegabar0)+sigmam1^2/2)/sigmam1;
    Gamma0   =   normcdf(xzx0-sigmam1)+omegabar0*(1-normcdf(xzx0));
    Gam_muG0 =   (1-mu)*normcdf(xzx0-sigmam1)+omegabar0*(1-normcdf(xzx0));
    
    muG0=Gamma0-Gam_muG0;
    
    % Change logncdf to normcdf: logncdf(x,y,z) = normcdf((log(x)-y)/z);
    %Fp1=logncdf(omegabarp1,-sigma0^2/2,sigma0);
    Fp1=normcdf((log(omegabarp1)+sigma0^2/2)/sigma0);
    
    % Change lognpdf to normpdf: lognpdf(x,y,z) = normpdf((log(x)-y)/z)/x/z;
    %dFp1=lognpdf(omegabarp1,-sigma0^2/2,sigma0);
    dFp1=normpdf((log(omegabarp1)+sigma0^2/2)/sigma0)/omegabarp1/sigma0;
    
    lambdap1=(1-Fp1)/(1-Fp1-mu*omegabarp1*dFp1);
    %[Gammap1,Gam_muGp1] = helpd(omegabarp1,mu,sigma0);
    xzxp1       =   (log(omegabarp1)+sigma0^2/2)/sigma0;
    Gammap1   =   normcdf(xzxp1-sigma0)+omegabarp1*(1-normcdf(xzxp1));
    Gam_muGp1 =   (1-mu)*normcdf(xzxp1-sigma0)+omegabarp1*(1-normcdf(xzxp1));
    
    %two efficiency conditions for CSV contracts
    ff(6)=(1-Gammap1)*ssp1+lambdap1*(ssp1*Gam_muGp1-1);
    ff(7)=(qm1*kbar0*(1+Rk0)*Gam_muG0/(n0*(1+Re0)))-(qm1*kbar0/n0)+1;
    %law of motion for net worth
    ff(8)=-np1+(gamma0/(pi0*muzstar0))*(Rk0-Re0-muG0*(1+Rk0))*kbar0*qm1+we+gamma0*(1+Re0)*n0/(pi0*muzstar0);
end
%
a0=rk*(exp(sigmaa*(u0-1))-1)/sigmaa;
ap1=rk*(exp(sigmaa*(up1-1))-1)/sigmaa;
%definition of rate of return on capital
ff(9)=Rk0 - ( ( (1-taukm1)*(u0*rk0-tauo0*a0)+(1-delta)*q0)*pi0/(upsil*qm1) + taukm1*delta - 1);

if entr > -1
    %ratio, excess reserves to value added in banking sector
    ff(10)=-ev0+((1-tau)*(mb0/(pi0*muzstar0))*(1-m0)-tau*(psil0*w0*l0+psik0*rk0*kbar0*u0/(muzstar0*upsil)))/ ...
        ((1-vl0)*((u0*kbar0/(muzstar0*upsil))^alpha)*(l0^(1-alpha)));
    if abs(rhobank) < .1e-9
        zit=ev0^(1-ksi0);
        zit1=(ev0^(-ksi0));
        zi=zit1;
    else
        zi=(ksi0*(ev0^(-rhobank))+(1-ksi0))^(1/rhobank);
        zit=(ksi0+(1-ksi0)*(ev0^rhobank))^((1-rhobank)/rhobank);
        zit1=zi^(1-rhobank);
    end
    
    hlbz0 = (1-alpha)*ksi0*xb0*(zit)*((kbar0*u0/(muzstar0*upsil*l0))^alpha);
    her0  = (1-ksi0)*xb0*zit1;
    herp1 = (1-ksip1)*xbp1*evp1^(-ksip1);
    %efficiency condition for inputs into banking sector
    ff(11)= R0*hlbz0/(1+tau*her0)-(1+psil0*R0)*w0;
    
    m10  = (mb0/(pi0*muzstar0))*(1-m0+chorse*dm0);
    m20  = psil0*w0*l0+psik0*rk0*kbar0*u0/(muzstar0*upsil);
    %erz0=(1-tau0)*m10-tau0*m20;
    erz0   = (1-tau)*(mb0/(pi0*muzstar0))*(1-m0)-tau*m20;
    %production function for deposits
    ff(12)= xb0*zi*erz0-(m10+m20);
    
    %efficiency condition for banks
    ff(13)= Ra0-(((1-tau)*her0-1)/(tau*her0+1))*R0;
end

%marginal utility of consumption (net of MIUF considerations)
ff(14)= uzc0-(muzstar0*zetac0/(c0*muzstar0-b*cm1))+b*beta*zetacp1/(cp1*muzstarp1-b*c0);

if entr > -1
    %first order condition for time deposits
    ff(15)= -zetac0*lambdaz0+(beta*zetacp1*lambdazp1/(muzstarp1*pip1))*(1+(1-taudp1)*Rep1);
end

if entr > -1
    if pref == 1%benchmark preferences
        aa0   = zetac0*v0*( (1+tauc0)*c0*((1/m0)^((1-chii0)*theta0)) * ((1/(1-m0))^((1-chii0)*(1-theta0))) * ((1/dm0)^chii0)    )^(1-sigmaq);
        aap1  = zetacp1*vp1*( (1+taucp1)*cp1 * ((1/mp1)^((1-chiip1)*thetap1)) * ((1/(1-mp1))^((1-chiip1)*(1-thetap1))) * (1/dmp1)^chiip1 )^(1-sigmaq);
    elseif pref == 0%ACEL preferences
        aa0   = zetac0*v0*( (1/m0)^((1-chii0)*theta0) * (1/(1-m0))^((1-chii0)*(1-theta0)) * (1/dm0)^chii0    )^(1-sigmaq);
        aap1  = zetacp1*vp1*(((1/mp1)^thetap1)*((1/(1-mp1))^(1-thetap1)))^(1-sigmaq);
    end

    %first order condition for currency
    HROB   = pim1*muzstarm1*mb0*m0/(mbm1*mm1);
    A      = sqrt(Hdoupr/2);
    H0     = exp(A*(HROB-muzstar*pi))+exp(-A*(HROB-muzstar*pi))-2;
    dH0    = A*exp(A*(HROB-muzstar*pi))-A*exp(-A*(HROB-muzstar*pi));
    HROBp1 = pi0*muzstar0*mbp1*mp1/(mb0*m0);
    dHp1   = A*exp(A*(HROBp1-muzstar*pi)) - A*exp(-A*(HROBp1-muzstar*pi));


    ff(16)= aa0*((theta0*(1-chii0)/m0)-(1-chii0)*(1-theta0)/(1-m0))*(((pi0*muzstar0)/mb0)^(2-sigmaq)) ...
        - zetac0*lambdaz0*(1-taud0)*Ra0 ...
        -zetac0*dH0*muzstar0*pim1*pi0*muzstarm1/(mbm1*mm1) +beta*zetacp1*dHp1*((muzstar0*pi0)^2)*mbp1*mp1/((mb0*m0)^2);

elseif entr == -1

    aa0   = v0*( (1+tauc0)*c0/m0 )^(1-sigmaq);

    ff(16)= aa0*(1/m0)*(((pi0*muzstar0)/mb0)^(2-sigmaq)) - lambdaz0*R0;

end

%first order condition for base
if entr > -1
    
    ff(17)= (  aap1*beta*(1-thetap1)*(1-chiip1)*(mbp1^(sigmaq-2))*((pip1*muzstarp1)^(1-sigmaq))/(1-mp1)  ) ...
        +(beta*zetacp1*lambdazp1/(pip1*muzstarp1))*(1+(1-taudp1)*Rap1) - zetac0*lambdaz0;
    
elseif entr <= -1
    
    ff(17)= (beta*zetacp1*lambdazp1/(pip1*muzstarp1))*(1+Rp1) - zetac0*lambdaz0;
    
end


if entr >= -1
    %first order condition for consumption
    if pref == 1%benchmark preferences
        ff(18)=uzc0-(aa0/c0)*((pi0*muzstar0/mb0)^(1-sigmaq))-zetac0*(1+tauc0)*lambdaz0;
    elseif pref == 0%ACEL preferences
        ff(18)=uzc0-zetac0*(1+tauc0)*lambdaz0;
    end
end

%resource constraint
if entr == 1
    d0=muG0*(1+Rk0)*qm1*kbar0/(muzstar0*pi0);
    ff(20)=Yz0-g0-c0-i0/muup0-tauo0*a0*kbar0/(muzstar0*upsil)-d0-bigtheta*(1-gamma0)*(np1-we)/gamma0;
elseif entr < 1
    ff(20)=Yz0-g0-c0-i0/muup0-tauo0*a0*kbar0/(muzstar0*upsil);    
end

ff(21)=kbarp1-(1-delta)*(kbar0/(muzstar0*upsil))-(1-S0)*i0;

if entr >= -1
    ff(22)=mbp1-mb0*(1+x0)/(pi0*muzstar0);
end

%monetary policy rule
y  = (c+i)/(1-etag);

if entr <= -1
    Re0=Rm1;
    Rep1=R0;
    Re=R;
end

if Taylor ==2

    if forward ==1
        if entr==1
            ff(23) = log(Rep1)-log(Re) - rhotil*(log(Re0) -log(Re)) -(1/Re)*(1-rhotil)*pi*( log(pitarget0) -log(pi) ) - (1/Re)*(1-rhotil)*aptil*pi*(log(pip1) - log(pitarget0)) - (1/(4*Re))*(1-rhotil)*aytil*( c/y*log(c0)+i/y*log(i0)-i/y*log(muup0)+g/y*log(g0)  - (c/y*log(c)+i/y*log(i)+g/y*log(g)) ) - (1/Re)*(1-rhotil)*amtil*muzstar*pi*( log(m30)-log(m3m1)+log(muzstar0)+log(pi0)  -log(muzstar*pi) )  - (1/Re)*(1-rhotil)*actil*muzstar*( log(btot0)-log(btotm1)+log(muzstar0)  -log(muzstar)) - (1/(400*Re))*xp0;
        elseif entr ==0
            ff(23) = log(Rep1)-log(Re) - rhotil*(log(Re0) -log(Re)) -(1/Re)*(1-rhotil)*pi*( log(pitarget0) -log(pi) ) - (1/Re)*(1-rhotil)*aptil*pi*(log(pip1) - log(pitarget0)) - (1/(4*Re))*(1-rhotil)*aytil*( c/y*log(c0)+i/y*log(i0)-i/y*log(muup0)+g/y*log(g0)  - (c/y*log(c)+i/y*log(i)+g/y*log(g)) ) - (1/Re)*(1-rhotil)*amtil*muzstar*pi*( log(m30)-log(m3m1)+log(muzstar0)+log(pi0)  -log(muzstar*pi) )  - (1/(400*Re))*xp0;
        elseif entr <=-1
            ff(23) = log(Rep1)-log(Re) - rhotil*(log(Re0) -log(Re)) -(1/Re)*(1-rhotil)*pi*( log(pitarget0) -log(pi) ) - (1/Re)*(1-rhotil)*aptil*pi*(log(pip1) - log(pitarget0)) - (1/(4*Re))*(1-rhotil)*aytil*( c/y*log(c0)+i/y*log(i0)-i/y*log(muup0)+g/y*log(g0)  - (c/y*log(c)+i/y*log(i)+g/y*log(g)) )  - (1/(400*Re))*xp0;
        end
    elseif forward ==0
        if entr==1
            ff(23) = log(Rep1)-log(Re) - rhotil*(log(Re0) -log(Re)) -(1/Re)*(1-rhotil)*pi*( log(pitarget0) -log(pi) ) - (1/Re)*(1-rhotil)*aptil*pi*(log(pi0) - log(pitarget0)) - (1/(4*Re))*(1-rhotil)*aytil*( c/y*log(c0)+i/y*log(i0)-i/y*log(muup0)+g/y*log(g0)  -(c/y*log(c)+i/y*log(i)+g/y*log(g))) - (1/Re)*(1-rhotil)*amtil*muzstar*pi*( log(m30)-log(m3m1)+log(muzstar0)+log(pi0)  -log(muzstar*pi) )  - (1/Re)*(1-rhotil)*actil*muzstar*( log(btot0)-log(btotm1)+log(muzstar0)  -log(muzstar)) - (1/(400*Re))*xp0;
        elseif entr ==0
            ff(23) = log(Rep1)-log(Re) - rhotil*(log(Re0) -log(Re)) -(1/Re)*(1-rhotil)*pi*( log(pitarget0) -log(pi) ) - (1/Re)*(1-rhotil)*aptil*pi*(log(pi0) - log(pitarget0)) - (1/(4*Re))*(1-rhotil)*aytil*( c/y*log(c0)+i/y*log(i0)-i/y*log(muup0)+g/y*log(g0)  -(c/y*log(c)+i/y*log(i)+g/y*log(g))) - (1/Re)*(1-rhotil)*amtil*muzstar*pi*( log(m30)-log(m3m1)+log(muzstar0)+log(pi0)  -log(muzstar*pi) )  - (1/(400*Re))*xp0;
        elseif entr <=-1
            ff(23) = log(Rep1)-log(Re) - rhotil*(log(Re0) -log(Re)) -(1/Re)*(1-rhotil)*pi*( log(pitarget0) -log(pi) ) - (1/Re)*(1-rhotil)*aptil*pi*(log(pi0) - log(pitarget0)) - (1/(4*Re))*(1-rhotil)*aytil*( c/y*log(c0)+i/y*log(i0)-i/y*log(muup0)+g/y*log(g0)  -(c/y*log(c)+i/y*log(i)+g/y*log(g)))  - (1/(400*Re))*xp0;
        end
    elseif forward ==-1
        if entr ==1
            ff(23) = log(Rep1)-log(Re) - rhotil*(log(Re0) -log(Re)) -(1/Re)*(1-rhotil)*pi*( log(pitarget0) -log(pi) ) - (1/Re)*(1-rhotil)*aptil*pi*(log(pim1) - log(pitarget0)) - (1/(4*Re))*(1-rhotil)*aytil*( c/y*log(c0)+i/y*log(i0)-i/y*log(muup0)+g/y*log(g0)  -(c/y*log(c)+i/y*log(i)+g/y*log(g))) - (1/Re)*(1-rhotil)*amtil*muzstar*pi*( log(m30)-log(m3m1)+log(muzstar0)+log(pi0)  -log(muzstar*pi) )  - (1/Re)*(1-rhotil)*actil*muzstar*( log(btot0)-log(btotm1)+log(muzstar0)  -log(muzstar)) - (1/(400*Re))*xp0;
        elseif entr ==0
            ff(23) = log(Rep1)-log(Re) - rhotil*(log(Re0) -log(Re)) -(1/Re)*(1-rhotil)*pi*( log(pitarget0) -log(pi) ) - (1/Re)*(1-rhotil)*aptil*pi*(log(pim1) - log(pitarget0)) - (1/(4*Re))*(1-rhotil)*aytil*( c/y*log(c0)+i/y*log(i0)-i/y*log(muup0)+g/y*log(g0)  -(c/y*log(c)+i/y*log(i)+g/y*log(g))) - (1/Re)*(1-rhotil)*amtil*muzstar*pi*( log(m30)-log(m3m1)+log(muzstar0)+log(pi0)  -log(muzstar*pi) )  - (1/(400*Re))*xp0;
        elseif entr <=-1
            ff(23) = log(Rep1)-log(Re) - rhotil*(log(Re0) -log(Re)) -(1/Re)*(1-rhotil)*pi*( log(pitarget0) -log(pi) ) - (1/Re)*(1-rhotil)*aptil*pi*(log(pim1) - log(pitarget0)) - (1/(4*Re))*(1-rhotil)*aytil*( c/y*log(c0)+i/y*log(i0)-i/y*log(muup0)+g/y*log(g0)  -(c/y*log(c)+i/y*log(i)+g/y*log(g))) - (1/(400*Re))*xp0;
        end
    end
    
end

if entr <= 0
    ff(24)=-zetac0*lambdaz0+lambdazp1*zetacp1*(beta/(pip1*muzstarp1))*(1+Rkp1);
end

if entr == -1
    ff(25)=psil0*w0*l0+psik0*rk0*u0*kbar0/(muzstar0*upsil)-(mb0/(pi0*muzstar0))*(1-m0);
end

if entr >-1
    %FOC for saving deposits in banking sector
    ff(26)= zetacp1*lambdazp1/(muzstarp1*pip1)*(Rep1-Rmp1 - chorse*Rp1/(herp1*tau+1));
    
    %FOC for saving deposits in household sector
    ff(27)= (beta*chiip1*aap1*((1/mbp1)^(2-sigmaq))*((pip1*muzstarp1)^(1-sigmaq))/dmp1) + beta*zetacp1*lambdazp1*(1+(1-taudp1)*Rmp1)/(pip1*muzstarp1) - zetac0*lambdaz0;

    % definition of scaled M3
    ff(28)= m30 - (mb0/(pi0*muzstar0))*(1+dm0) - psil0*w0*l0 - psik0*rk0*u0*kbar0/(muzstar0*upsil) ;
end

if entr ==1
    %definition of scaled total loans
    ff(30)= btot0 -psil0*w0*l0 -psik0*rk0*u0*kbar0/(muzstar0*upsil) - (qm1*kbar0-n0)/(pi0*muzstar0);
end


%the following equations, (32) - (38), correspond to (31)-(37) in 'Dale' manuscript.

pit0  = (pitarget0^iota1) * (pim1^iota2) * pibar^(1-iota1-iota2) ;
pitp1 = (pitargetp1^iota1) * (pi0^iota2) * pibar^(1-iota1-iota2) ;

a0   = ( (1 - xip*(pit0/pi0)^(1/(1-lambdaf0)) ) / (1-xip) );
ap1  = ( (1 - xip*(pitp1/pip1)^(1/(1-lambdafp1)) ) / (1-xip) );
aa0  = a0 ^ lambdaf0;
pow  = lambdaf0 / (1-lambdaf0);

%dynamic equation for price distortions
ff(32)= pstar0 - ((1-xip)*aa0 + xip*(((pit0/pi0)*pstarm1)^pow) )^(1/pow);

ptwage0 = (pitarget0^iotaw1) * (pim1^iotaw2) * (pibar^(1-iotaw1-iotaw2));
ptwagep1 = (pitargetp1^iotaw1) * (pi0^iotaw2) * (pibar^(1-iotaw1-iotaw2));

piw0   = pi0*muzstar0*w0/wm1;
piwp1  = pip1*muzstarp1*wp1/w0;
cc0    = (ptwage0/piw0) * (muzstar^(1-iotamu)) * (muzstar0^iotamu);
ccp1   = (ptwagep1/piwp1) * (muzstar^(1-iotamu)) * (muzstarp1^iotamu);
b0     = (1 - xiw * ( cc0 ^ (1/(1-lambdaw)) ) ) / (1-xiw) ;
bp1    = (1 - xiw * ( ccp1 ^ (1/(1-lambdaw)) ) ) / (1-xiw) ;
bb0    = b0 ^ lambdaw;
poww   = lambdaw/(1-lambdaw);

%dynamic equation for wage distortions
ff(33)=  wstar0 - ( ((1-xiw)*bb0 + xiw*( (cc0*wstarm1)^poww ) )^(1/poww) );

Kp0    =  Fp0* (a0^(1-lambdaf0));
Kpp1   =  Fpp1* (ap1^(1-lambdafp1));

Kw0    =  ( b0   ^ (1-lambdaw*(1+sigmaL)) ) * w0   * Fw0/psiL;
Kwp1   =  ( bp1 ^ (1-lambdaw*(1+sigmaL)) ) * wp1 * Fwp1/psiL;

ff(34)=  zetac0*lambdaz0*Yz0 + ( (pitp1/pip1) ^ (1/(1-lambdafp1)) ) * beta * xip * Fpp1 - Fp0;

ff(35)= zetac0*lambdaf0*lambdaz0*Yz0*s0 + beta * xip *  ( (pitp1/pip1) ^ (lambdafp1/(1-lambdafp1)) ) * Kpp1 - Kp0;

ff(36)= zetac0*((wstar0^(lambdaw/(lambdaw-1)))*h0*(1-taul0)*lambdaz0/lambdaw) ...
    + beta * xiw * (muzstar^((1-iotamu)/(1-lambdaw))) ...
    * ( (ptwagep1^(1/(1-lambdaw))) / pip1 ) * ( (1/piwp1) ^ (lambdaw/(1-lambdaw)) ) ... 
    * (muzstarp1^(iotamu/(1-lambdaw)-1)) * Fwp1 - Fw0;

ff(37)= zetac0*((wstar0^(lambdaw/(lambdaw-1))*h0)^(1+sigmaL))*zeta0 + beta * xiw * ...
        (( (ptwagep1/piwp1) * (muzstar^(1-iotamu)) * (muzstarp1^iotamu) )  ...
        ^ (lambdaw*(1+sigmaL)/(1-lambdaw))) * Kwp1 - Kw0;

powww  = lambdaw*(1+sigmaL)/(1-lambdaw);
b0     = (1 - xiw * ( cc0 ^ (1/(1-lambdaw)) ) ) / (1-xiw) ;
bb0    = b0^(lambdaw*(1+sigmaL));
ff(38)= wplus0 - ( ((1-xiw)*bb0 + xiw*( (cc0*wplusm1)^powww ) )^(1/powww) );
