// this is the real business cycle version of the model with sticky prices
// when lambdaf is close to unity, the Ramsey solution to the sticky price model should coincide with the one here.


var cU, hU, kU, wU, lambdazU, qU, iU, epsilU, Ux;
varexo SIGNAL epx;

//PARAMETERS
parameters sigmaLXUU, betaUU, psiLXUU, zetaUU, alphaUU, deltaUU, xipUU,
      iota1UU, iota2UU, phiUU, pibarUU,aptilUU, aytilUU,  rhotilUU,    
      xUU,lambdafUU,epsilUU,etagUU,pitargetUU,cUU,RXUU,piUU,hUU,kUU,pstarUU,FpXUU,UtilUU,WelfUU,nbeta,
      rhoUU, bet, lambdazUU,qUU,iUU,SdouprXUU,bUU,
      bUUU, SdouprUUU, betaUUU, deltaUUU, alphaUUU, psiLXUUU, sigmaLXUUU, rhoUUU,wUU,lambdazUU,muzstarUU,muzstarUUU;

      
 
// %---Household
lambdawUU     =   1;
sigmaLXUU      =   1;
zetaUU        =   1.000; 
betaUU        =   0.99;  
nbeta         =   betaUU; 
xiwUU         =   0;
bUU           =   0.63; 
psiLXUU        =   109.8157082828666;  
zetacUU       =   1;
iotaw1UU      =   0.0;
iotaw2UU      =   0.13;
iotamuUU      =   0;
tauoUU        =   1;
// %----- Capital Producer
sigmaaUU      =   12.3;
SdouprXUU      =   5.1;
etaiUU        =   2; 
muupUU        =   1; 
zetaiUU       =   1;
// %---- Goods Producers
muzstarUU     =   1.0136^(.25);
upsilUU       =   1; 
lambdafUU     =   1.;
alphaUU       =   0.3987246;
psikUU        =   0;
psilUU        =   0;
deltaUU       =   0.025;
epsilUU       =   1;
xipUU         =   0.75;
iota1UU       =   0.0;
iota2UU       =   0.6;
// %---- Taxes and government consumption
etagUU        =   0;
taukUU        =   0;
taulUU        =   0;
taucUU        =   0;
// %---- Monetary Policy
xUU           =   0.009152;

// %wage/price setting index parameter, automatically set to steady state inflation when not optimizing policy
pibarUU       =   (1+xUU)/muzstarUU;
aptilUU       =  1.95 ;
aytilUU       =  0.18 ;
rhotilUU      =  0.81 ;
rhoUU         =  0.8;

[hUU,kUU,cUU,lambdazUU,wUU] = sstaterbcadjhabit(betaUU,deltaUU,alphaUU,psiLXUU,sigmaLXUU,bUU,muzstarUU);
epsilUU=1;
qUU=1;

model;


// intertemporal Euler equation (aapp) 
-lambdazU + (lambdazU(+1)*betaUU/(muzstarUU*qU))*(alphaUU*epsilU(+1)*(muzstarUU * hU(+1) /kU)^(1-alphaUU) + qU(+1)*(1-deltaUU) ); 

// resource constraint (resource) 
cU + iU - epsilU * ( kU(-1)/muzstarUU )^alphaUU*hU^(1-alphaUU); 
 
// capital accumulation equation (capital) 
kU - ((1-deltaUU)/muzstarUU)*kU(-1) - (1 - (muzstarUU^2)*(SdouprXUU/2)*(iU/iU(-1)-1)^2)*iU; 

// investment equation (invest) 
lambdazU*qU*(1-(muzstarUU^2)*(SdouprXUU/2)*(iU/iU(-1)-1)^2-(muzstarUU^2)*SdouprXUU*(iU/iU(-1)-1)*iU/iU(-1)) - lambdazU 
+ betaUU*lambdazU(+1)*qU(+1)*(muzstarUU^2)*SdouprXUU*(iU(+1)/iU-1)*(iU(+1)/iU)^2; 
 
// lambdaz (lamz) 
lambdazU - muzstarUU/(muzstarUU*cU-bUU*cU(-1)) + bUU*betaUU/(muzstarUU*cU(+1)-bUU*cU); 

// intratemporal euler equation
psiLXUU*(hU^sigmaLXUU) - lambdazU*(1-alphaUU)*epsilU*(kU(-1)/(muzstarUU*hU))^alphaUU;

// real, scaled, wage
wU - psiLXUU*(hU^sigmaLXUU)/lambdazU;

// law of motion of exogenous shock
log(epsilU) = rhoUU*log(epsilU(-1))+ Ux(-12) + epx;
Ux = SIGNAL;

end;


initval;
kU=kUU;
hU=hUU;
cU=cUU;
wU=wUU;
lambdazU=lambdazUU;
epsilU=epsilUU;
qU=1;
iU=deltaUU*kUU;
uX=0;
end;

shocks;
var SIGNAL;
stderr 1;
var epx;
stderr .00001;
end;

steady;

// stoch_simul(dr_algo=0,periods=1000,simul_seed=1,irf=100);
stoch_simul(order=1,irf=100) cU hU kU epsilU qU iU wU;

crbc=cU_SIGNAL;
hrbc=hU_SIGNAL;
irbc=iU_SIGNAL;
qrbc=qU_SIGNAL;
krbc=kU_SIGNAL;
wrbc=wU_SIGNAL;
Epsrbc=epsilU_SIGNAL;
  
for ii = 13:24 
    crbc(ii)=crbc(ii)-cU_epx(ii-12);
    hrbc(ii)=hrbc(ii)-hU_epx(ii-12);
    irbc(ii)=irbc(ii)-iU_epx(ii-12);
    qrbc(ii)=qrbc(ii)-qU_epx(ii-12);
    krbc(ii)=krbc(ii)-kU_epx(ii-12);
    wrbc(ii)=wrbc(ii)-wU_epx(ii-12);
    Epsrbc(ii)=Epsrbc(ii)-epsilU_epx(ii-12);
end 

bUUU=bUU;
SdouprXUUU=SdouprXUU;
betaUUU=betaUU;
deltaUUU=deltaUU;
alphaUUU=alphaUU;
psiLXUUU=psiLXUU;
sigmaLXUUU=sigmaLXUU;
rhoUUU=rhoUU;
muzstarUUU=muzstarUU

save rbc crbc hrbc irbc qrbc krbc Epsrbc wrbc bUUU SdouprXUUU betaUUU deltaUUU alphaUUU psiLXUUU sigmaLXUUU rhoUUU muzstarUUU

%dr_algo=0, perturbation approach a la Schmitt-Grohe-Uribe
%order = 2, second order approximation, default
%irf, # of periods for impulse responses
%variables next to parentheses means these are the variables you want

tt=1:24;
subplot(331)
plot(tt,crbc(tt)/cUU)
title('consumption')
axis tight
subplot(332)
plot(tt,hrbc(tt)/hUU)
title('hours worked')
subplot(333)
plot(tt,irbc(tt)/(deltaUU*kUU))
title('investment')
axis tight
subplot(334)
plot(tt,qrbc(tt)/qUU)
title('price of capital')
axis tight
subplot(335)
plot(tt,krbc(tt)/kUU)
title('capital stock')
axis tight
subplot(336)
plot(tt,Epsrbc(tt))
title('shock')
axis tight
subplot(337)
plot(tt,wrbc(tt)/wUU)
title('real wage')
axis tight

