%Instructions for running the optimal policy code
%
%General notational observation: variables end in U; constants end in UU;
%the name of an object which has any part of it capitalized has an X before
%the first U.
%
%Make sure that dynare and Andy Levin's code are in your MATLAB path. In my
%computer, I enter this at the MATLAB prompt:
%
%       addpath('c:\projects\solve\dynare')
%       addpath('c:\projects\solve\levin')
%
%The basic model with policy rule is in newsimplemodel.mod
%This *.mod file must be adjusted depending on whether you want to run it
%with the exogenous policy, or if you want optimal policy. With exogenous
%policy, you must have a single var statement only. To get it ready for
%optimal policy, you have to have two var statements, one with the
%variables to be optimized in the Ramsey problem, and one with the rest.
%The var statement with the variables to be optimized in the 
%Ramsey problem must be preceded by 
%
%         // Endogenous variables
%
%The var statement with the other 'endogenous' variables must be preceded
%by
%         // Exogenous variables
%
%Whether you want to analyze optimal policy or exogenous policy, you must
%have, among the equations of the model, an equation that corresponds
%to exogenous policy, preceded by
%
%        // Monetary Policy Rule
%
%To analyze the model with exogenous policy, make sure the *.mod file is
%set up correctly, and then at the MATLAB command prompt, type 
%
% dynare newsimplemodel
%
%To run optimal policy, you need to apply Levin's translation code first.
%This requires making sure the *.mod file is set up correctly. Then,
%at the Matlab prompt type
%
%           infilename  = 'newsimplemodel';
%           outfilename = 'newsimplemodel_OUT';
%           get_ramsey
%
%this replaces the monetary policy rule with the first order
%conditions associated with the Lagrangian representation of the Ramsey
%problem.
%
%Once the translation is complete, things need to be done to the new
%program, newsimplemodel_OUT.mod, so that it can be run by Dynare.
%
%1. three var statements must be merged - the ones for endogenous and
%exogenous variables, and the one for the multipliers.
%
%2. the call to the steady state program must be replaced by a call to
%ssnew. This computes the steady state of the Ramsey problem.
%The strategy for doing this is to specify a steady state money growth
%rate, and then compute the N-1 other variables that solve the steady state
%equilibrium conditions. Then, the steady state multipliers are found which
%solve the N first order conditions of the Ramsey problem. This solution is
%only approximate because there are only N-1 multipliers for this. The
%variable, x, is varied until all N first order conditions of the Ramsey
%problem are satisfied in steady state.
%
%The program which computes the steady state of the N-1 equilibrium
%conditions is called sstate.m. The program which computes the steady state
%of the multipliers is Levin's endogenously generated code, newsimplemodel_out_lmss.m
%
%make sure ssnew is after the parameters statement with the multipliers
%
%3.%newsimplemodel_out_lmss.m must be adjusted in three ways:
%
%a. comment out the global statements at the start of the code.
%b. do a global replacement of the object, _SS, with U.
%c. it is a good idea to remove the error check,
% 
% if errcheck > 1e-08, 
%   disp('Warning: steady states of lagrange multipliers cannot be accurately determined'); 
%   disp(['         errcheck = ',num2str(errcheck,12)]); 
% end; 
% 
%this check need not be put anywhere else because it appears at the end of ssnew. 
%leaving the check in newsimplemodel_out_lmss.m is a nuisance because 
%this program is evaluated repeatedly in the process of computing a steady
%state. Until the steady state is found, the check generates a warning.
%
%The program, newsimplemodel_out.mod has to have the same replacement as in
%b above, replace _SS with U everywhere.
%
