// model with v > 0

var SXU, SpXU, lambdazU, iU, cU, piU rkU, PkXU, RXU, kU, zU, pstarU, sU, lU, FpXU, KpXU;
varexo epsilon mshock;

parameters beta,b,alpha,delta,rho,lambdaf,xip,iota,pibar,rhoi,alphapi,alphay,v,sigmaL,sigmaq,a,psiL;
parameters rk_ss,k_ss,l_ss,c_ss,w_ss,phi,lambdaz_ss,y_ss,R_ss,s_ss,m_ss, Fp_ss;

[beta,b,alpha,delta,rho,lambdaf,xip,iota,pibar,rhoi,alphapi,alphay,sigmaL,sigmaq,a,psiL] = params;
v=.000000;

[rk_ss,k_ss,l_ss,c_ss,w_ss,phi,lambdaz_ss,y_ss,R_ss,s_ss,m_ss,Fp_ss] = ssteady(lambdaf,alpha,delta,sigmaL,sigmaq,b,pibar,v,beta,psiL,xip);

model;
// Monetary Policy Rule
RXU-rhoi*RXU(-1)-(1-rhoi)*(pibar/beta-1+alphapi*(piU(+1)-pibar)+alphay*log((cU+iU)/y_ss)) - mshock;
// definitions of adjustment costs:
SXU=(a/2)*(iU/iU(-1)-1)^2;
SpXU=a*(iU/iU(-1)-1);
// equation (1)
lambdazU - 1/(cU-b*cU(-1)) + beta*b/(cU(+1)-b*cU);
// lambdazU - 1/(cU-b*cU(-1)) + beta*b/(cU(+1)-b*cU) + v*(1/mU)^(1-sigmaq)/cU^sigmaq; case of P(t)c(t)/M(t+1) in utility function
// equation (2)
1 - PkXU*(1-SXU-SpXU*iU/iU(-1)) - beta*(lambdazU(+1)/lambdazU)*PkXU(+1)*SpXU(+1)*(iU(+1)/iu)^2;
// equation (3)
PkXU - (lambdazU(+1)/lambdazU)*beta*(rkU(+1) + PkXU(+1)*(1-delta));
// equation (5) 
lambdazU - beta*(lambdazU(+1)/piU(+1))*(1+RXU);
// equation (6)
(1-delta)*kU(-1)+(1-SXU)*iU-kU;
// equation (7)
cU+iU - (pstarU^(lambdaf/(lambdaf-1)))*(zU*(kU(-1)^alpha)*(lU^(1-alpha))-phi);
// equation (8)
psiL*lU^sigmaL-lambdazU*sU*zU*(1-alpha)*(kU(-1)/lU)^alpha;
// equation (9)
rkU-sU*zU*alpha*(lU/kU(-1))^(1-alpha);
// equation (10)
pstarU - ( (1-xip) * ( ( 1-xip*((piU(-1)^iota*pibar^(1-iota))/piU)^(1/(1-lambdaf)) )/(1-xip) )^lambdaf 
  + xip*(piU(-1)^iota*pibar^(1-iota)*pstarU(-1)/piU)^(lambdaf/(1-lambdaf)))^((1-lambdaf)/lambdaf);  
// equation (11)
lambdazU*(cU+iU)+(piU^iota*pibar^(1-iota)/piU(+1))^(1/(1-lambdaf))*beta*xip*FpXU(+1)-FpXU;
// equation (12)
lambdaf*lambdazU*(cU+iU)*sU+beta*xip*((piU^iota*pibar^(1-iota)/piU(+1))^(lambdaf/(1-lambdaf)))*KpXU(+1)-KpXU;
// equation (13)
(KpXU/FpXU)^(1/(1-lambdaf)) - (1-xip*((piU(-1)^iota*pibar^(1-iota)/piU)^(1/(1-lambdaf))))/(1-xip); 
// law of motion of shock
log(zU) = rho*log(zU(-1)) + epsilon;
end;

initval;

FpXU=Fp_ss;
KpXU=Fp_ss;
lU=l_ss;
sU=1/lambdaf;
pstarU=1;
kU=k_ss;
zU=1;
SXU=0;
SpXU=0;
lambdazU=lambdaz_ss;
PkXU=1;
iU=delta*k_ss;
cU=c_ss;
piU=pibar;

rkU=rk_ss;
RXU=R_ss;
end;

steady;

shocks;
var epsilon; stderr 1.;
var mshock; stderr .01;
end;

stoch_simul(order=1,irf=10) cU lU zU PkXU iU piU RXU;

figure
subplot(3,3,1)
plot(cU_epsilon/c_ss)
title('Consumption')
ylabel('percent deviation from steady state')
axis tight
subplot(3,3,2)
plot(iU_epsilon/(delta*k_ss))
title('investment')
ylabel('percent deviation from steady state')
axis tight
subplot(3,3,3)
plot(lU_epsilon/l_ss)
title('hours worked')
ylabel('percent deviation from steady state')
axis tight
subplot(3,3,4)
plot(PkXU_epsilon)
title('price of capital')
ylabel('percent deviation from steady state')
axis tight

subplot(3,3,6)
plot((RXU_epsilon+R_ss)*4)
title('nominal rate of interest (APR)')
ylabel('percent')
axis tight
subplot(3,3,7)
plot(zU_epsilon)
title('state of technology')
ylabel('percent deviation from ss')
axis tight
subplot(3,3,8)
plot(4*(piU_epsilon+pibar-1))
title('rate of inflation (APR)')
ylabel('percent, net')
axis tight
str=['Response to Technology Shocks, \alpha_{\pi} = ',num2str(alphapi),' velocity = ',num2str(y_ss/m_ss)];
hout=suptitle(str);