function [rk,k,l,c,w,phi,lambdaz,y,R,s,m,Fp] = steady(lambdaf,alpha,delta,sigmaL,sigmaq,b,pibar,v,beta,psiL,xip)
rk=(1/beta)-(1-delta);

lk=(lambdaf*rk/alpha)^(1/(1-alpha));

ck=((lk^(1-alpha))/lambdaf) - delta;

w = (1/( lambdaf * ((1/(1-alpha))^(1-alpha)) * ((1/alpha)^alpha) * (rk^alpha) ))^(1/(1-alpha));

a0=ck*psiL*lk^sigmaL/w;
a1=(ck*pibar)^(1-sigmaq);
a2=(1-beta*b)/(1-b);
b0=(psiL*lk^sigmaL/w)*(1-beta/pibar);
b1=beta*(pibar*ck)^(1-sigmaq);

%set up an interval in which to search for the steady state capital stock
k=(a2/a0)^(1/(1+sigmaL));
xx=[.001 2*k];

%verify that the interval contains a solution:
ff1= findk(xx(1),sigmaL,sigmaq,v,a0,a1,a2,b0,b1);
ff2= findk(xx(2),sigmaL,sigmaq,v,a0,a1,a2,b0,b1);
if ff1*ff2 > 0
    error('fatal (steady) initial interval for capital does not bracket steady state, try widening the interval')
end
options=optimset('disp','off');
[x,fval,exitflag]=fzero(@findk,xx,options,sigmaL,sigmaq,v,a0,a1,a2,b0,b1);

[ff] = findk(x,sigmaL,sigmaq,v,a0,a1,a2,b0,b1);
if abs(imag(ff)) > .1e-10 | abs(ff) > .1e-10 | abs(imag(x)) > .1e-10
    error('fatal (steady) bad steady state capital stock')
end
k=x;
mk=(v*b1/(b0*k^(1+sigmaL)))^(1/(2-sigmaq));
m=mk*k;
c=ck*k;
l=lk*k;
y=k^alpha*l^(1-alpha)/lambdaf;
R=pibar/beta-1;
s=1/lambdaf;

lambdaz=psiL*l^sigmaL/w;
Fp=lambdaz*y/(1-xip*beta);
phi=((lambdaf-1)/lambdaf)*k^alpha*l^(1-alpha);


%now, check that the steady state is correct....

%resource constraint:
err(1)=c+delta*k-k^alpha*l^(1-alpha)+phi;

%marginal utility of consumption:
if abs(v) > .1e-10
    err(2)=lambdaz-(1-beta*b)/(c*(1-b))+v*(pibar/m)^(1-sigmaq)/(c^sigmaq);
else
    err(2)=lambdaz-(1-beta*b)/(c*(1-b));
end

%intertemporal equation
if abs(v) > .1e-10
    err(3)=-lambdaz+beta*lambdaz/pibar+v*beta*(pibar/m)^(1-sigmaq)*c^(1-sigmaq)/m;
else
    err(3)=-lambdaz+beta*lambdaz/pibar;
end 

%intratemporal
err(4)=psiL*l^sigmaL/lambdaz-w;

%efficiency conditions of firm
err(5)=s-((1/(1-alpha))^(1-alpha))*((1/alpha)^alpha)*rk^alpha*w^(1-alpha);
err(6)=s-rk/(alpha*(l/k)^(1-alpha));
err(7)=s-1/lambdaf;

if abs(err) > .1e-12
    error('fatal (steady) check on steady state failed')
end