// this is the real business cycle version of the model with sticky prices
// when lambdaf is close to unity, the Ramsey solution to the sticky price model should coincide with the one here.

var cU, hU, kU, epsilU;

varexo xp epx;

//PARAMETERS
parameters sigmaLXUU, betaUU, psiLXUU, zetaUU, alphaUU, deltaUU, xipUU,
      iota1UU, iota2UU, phiUU, pibarUU,aptilUU, aytilUU,  rhotilUU,    
      xUU,lambdafUU,epsilUU,etagUU,pitargetUU,cUU,RXUU,piUU,hUU,kUU,pstarUU,FpXUU,UtilUU,WelfUU,nbeta,
      rhoUU, bet;
      
 
// %---Household
lambdawUU     =   1;
sigmaLXUU      =   1;
zetaUU        =   1.000; 
betaUU        =   0.99;  
nbeta         =   betaUU; 
xiwUU         =   0;
bUU           =   0; 
psiLXUU        =   109.8157082828666;  
zetacUU       =   1;
iotaw1UU      =   0.0;
iotaw2UU      =   0.13;
iotamuUU      =   0;
tauoUU        =   1;
// %----- Capital Producer
sigmaaUU      =   12.3;
SdouprXUU      =   5.1;
etaiUU        =   2; 
muupUU        =   1; 
zetaiUU       =   1;
// %---- Goods Producers
muzstarUU     =   1;
upsilUU       =   1; 
lambdafUU     =   1.;
alphaUU       =   0.3987246;
psikUU        =   0;
psilUU        =   0;
deltaUU       =   0.025;
epsilUU       =   1;
xipUU         =   0.75;
iota1UU       =   0.0;
iota2UU       =   0.6;
// %---- Taxes and government consumption
etagUU        =   0;
taukUU        =   0;
taulUU        =   0;
taucUU        =   0;
// %---- Monetary Policy
xUU           =   0.009152;

// %wage/price setting index parameter, automatically set to steady state inflation when not optimizing policy
pibarUU       =   (1+xUU)/muzstarUU;
aptilUU       =  1.95 ;
aytilUU       =  0.18 ;
rhotilUU      =  0.81 ;
rhoUU         =  0.8;

[hUU,kUU,cUU] = sstaterbc(betaUU,deltaUU,alphaUU,psiLXUU,sigmaLXUU);
epsilUU=1;

model;

// intertemporal Euler equation
-1/cU + (betaUU/cU(+1))*(alphaUU*epsilU(+1)*(hU(+1)/kU)^(1-alphaUU)+1-deltaUU);

// resource constraint
cU + kU - (1-deltaUU)*kU(-1) - epsilU * ( kU(-1))^alphaUU  * hU^(1-alphaUU);

// intratemporal euler equation
psiLXUU*(hU^sigmaLXUU)*cU-(1-alphaUU)*epsilU*(kU(-1)/hU)^alphaUU;

// law of motion of exogenous shock
log(epsilU) = rhoUU*log(epsilU(-1))+ epx;

end;


initval;
kU=kUU;
hU=hUU;
cU=cUU;
epsilU=epsilUU;
end;

shocks;
var xp;
stderr 1;
var epx;
stderr 1;
end;

steady;

// stoch_simul(dr_algo=0,periods=1000,simul_seed=1,irf=100);
stoch_simul(order=1,irf=100) cU hU kU epsilU;
cUrbc_epx=cU_epx;
hUrbc_epx=hU_epx;
kUrbc_epx=kU_epx;
epsilUrbc_epx=epsilU_epx;

save rbc cUrbc_epx hUrbc_epx kUrbc_epx epsilUrbc_epx

%dr_algo=0, perturbation approach a la Schmitt-Grohe-Uribe
%order = 2, second order approximation, default
%irf, # of periods for impulse responses
%variables next to parentheses means these are the variables you want